@@ -24,10 +24,12 @@
 #include "git2/diff.h"
 #include "git2/errors.h"
 #include "git2/filter.h"
+#include "git2/global.h"
 #include "git2/graph.h"
 #include "git2/ignore.h"
 #include "git2/index.h"
 #include "git2/indexer.h"
+#include "git2/mailmap.h"
 #include "git2/merge.h"
 #include "git2/message.h"
 #include "git2/net.h"
@@ -39,7 +41,7 @@
 #include "git2/pack.h"
 #include "git2/patch.h"
 #include "git2/pathspec.h"
-#include "git2/push.h"
+#include "git2/proxy.h"
 #include "git2/rebase.h"
 #include "git2/refdb.h"
 #include "git2/reflog.h"
@@ -56,10 +58,11 @@
 #include "git2/status.h"
 #include "git2/submodule.h"
 #include "git2/tag.h"
-#include "git2/global.h"
 #include "git2/transport.h"
+#include "git2/transaction.h"
 #include "git2/tree.h"
 #include "git2/types.h"
 #include "git2/version.h"
+#include "git2/worktree.h"
 
 #endif
