@@ -205,13 +205,13 @@ $def .= ' -DGIT_USE_STAT_MTIM_NSEC -DGIT_USE_NEC';
 
 if ($is_gcc) {
 	# gcc-like compiler
-	$ccflags .= ' -Wall -Wno-unused-variable';
+	$ccflags .= ' -Wall -Wno-unused-variable -Wno-pedantic -Wno-deprecated-declarations';
 
 	# clang compiler is pedantic!
 	if ($is_osx) {
 		# clang masquerading as gcc
 		if ($Config{gccversion} =~ /LLVM/) {
-			$ccflags .= ' -Wno-deprecated-declarations -Wno-unused-const-variable -Wno-unused-function';
+			$ccflags .= ' -Wno-unused-const-variable -Wno-unused-function';
 		}
 
 		# Secure transport (HTTPS)
@@ -225,7 +225,7 @@ if ($is_gcc) {
 
 	# building with a 32-bit perl on a 64-bit OS may require this (supported by cc and gcc-like compilers,
 	# excluding some ARM toolchains)
-	if ($Config{ptrsize} == 4 && $Config{archname} !~ /^armv/) {
+	if ($Config{ptrsize} == 4 && $Config{archname} !~ /arm/) {
 		$ccflags .= ' -m32';
 	}
 } elsif ($is_sunpro) {
@@ -309,6 +309,14 @@ sub MY::c_o {
 .c\$(OBJ_EXT):
 	\$(CCCMD) \$(CCCDLFLAGS) "-I\$(PERL_INC)" \$(PASTHRU_DEFINE) \$(DEFINE) \$*.c $out_switch\$@
 };
+
+	if ($is_gcc) {
+		# disable parallel builds
+		$line .= qq{
+
+.NOTPARALLEL:
+};
+	}
 	return $line;
 }
 
@@ -345,6 +353,9 @@ $WriteMakefileArgs{OBJECT}  .= ' ' . join ' ', @objs;
 $WriteMakefileArgs{dynamic_lib} = {
 	OTHERLDFLAGS => $otherldflags
 };
+$WriteMakefileArgs{clean} = {
+	FILES => "*.inc"
+};
 
 unless (eval { ExtUtils::MakeMaker->VERSION(6.56) }) {
 	my $br = delete $WriteMakefileArgs{BUILD_REQUIRES};
